package edu.wpi.ejutr.actions;

import edu.wpi.ejutr.EjutrPlugin;
import edu.wpi.ejutr.file_operations.EJUTRFileWriter;
import edu.wpi.ejutr.logger.EJUTRTargetFileInfo;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IViewActionDelegate;
import edu.wpi.ejutr.ebob_observer.*;

public class StopRecording implements IObjectActionDelegate, IViewActionDelegate {

	private IWorkbenchPart targetPart;
	
	/**
	 * Constructor for Action1.
	 */
	public StopRecording() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		Shell shell = new Shell();
		
		EJUTRTargetFileInfo fileInfo = EJUTRTargetFileInfo.getInstance();
		System.out.println(fileInfo.getTestContainer());
		if(fileInfo.getTestContainer() == null) {
			MessageDialog.openError(new Shell(), "Not Recording",
			"There is no test method currently being recorded");
		}
		else {
			EBOBListener eList = EBOBListener.getInstance();
			
			// confirmation dialog - three different kinds of stops
			//		1 - OK, dump it all to a file
			//		2 - Cancel, stop recording and clear log
			//		3 - Cancel, keep recording
			String[] buttonLables = new String[3];
			buttonLables[0] = "Yes - Stop and Save Recording";			
			buttonLables[1] = "Yes - Stop and Discard Recording";
			buttonLables[2] = "No - Continue Recording";
			
			MessageDialog stopDialog = new MessageDialog(shell,"Stop Recording",null,
				"Do you wish to stop Recording?", MessageDialog.QUESTION,
				buttonLables,0);
			stopDialog.setBlockOnOpen(true);
			//get the buttonlable index of the button that was pressed
			int response = stopDialog.open();  
			
		
			if(response == 0) {  //stop recording and dump to file
				String containerName = fileInfo.getTestContainer();
				String fileName = fileInfo.getTestFileName();
				String methodName = fileInfo.getTestMethodName();
				
				IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
				IContainer container = (IContainer) root.findMember(new Path(containerName));
				IFile file = container.getFile(new Path(fileName));
				
				EJUTRFileWriter fw = new EJUTRFileWriter(file);
				fw.dumpTo(methodName);
				fileInfo.clear();		
				eList.detach();
				
			}
			else if (response == 2) {  //continue recording
				//do nothing
			}
			else if (response == 1) {  //discard recording
				fileInfo.clear();		
				eList.detach();
			}
		}		
		
		
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	public void init(IViewPart view) {
		targetPart = view;
		
	}

}
