package edu.wpi.ejutr.wizards;

import edu.wpi.ejutr.logger.*;
import javax.swing.*;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.swt.events.*;

/**
 * The physical page displayed by the AssertionWizard.
 * @author bigben - <a href="mailto:bigben@wpi.edu">bigben@wpi.edu</a>
 */
public class AssertionWizardPage extends WizardPage {
	
	private String retType;
	private String methodCall;
	
	private Text lhsText;
	private Text compText;
	private Text rhsText;
	private Text previewText;
	private Button trueButton;
	private Button falseButton;
	private Button checkBox;
	private Button check2;
	
	private Combo operatorBox;
	
	private String lhs;
	private String rhs;
	private String comp;
	private String expected;
	private boolean expectedResult;
	
	private boolean isEnabled = false;
		
	public AssertionWizardPage(String retType, String methodCall) {
		super("wizardPage");
		setTitle("EJUTR - Assertion Wizard");
		setDescription("This wizard allows the addition of JUnit assertions.");
		this.retType = retType;
		this.methodCall = methodCall;
	}
	
	public void dispose() {
		//lhs = lhsText.getText();
		//comp = compText.getText();
		//rhs = rhsText.getText();
		//expectedResult = trueButton.getSelection();
	}
	
	public void createControl(Composite parent) {
		// layout stuff goes here	
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();		
		layout.numColumns = 2;
		layout.verticalSpacing = 5;
		container.setLayout(layout);
		
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		
		Label firstLabel = new Label(container, SWT.NULL);
		firstLabel.setText("Enable assertion for this method call: ");
		
		checkBox = new Button(container, SWT.CHECK);
		checkBox.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {				
			}
			public void widgetSelected(SelectionEvent e) {
				isEnabled = !isEnabled;
				lhsText.setEnabled(isEnabled);
				//compText.setEnabled(isEnabled);
				operatorBox.setEnabled(isEnabled);
				rhsText.setEnabled(isEnabled);
				check2.setEnabled(isEnabled);
				previewText.setEnabled(isEnabled);
				//trueButton.setEnabled(isEnabled); // causes exceptions
				//falseButton.setEnabled(isEnabled);
				//expectedText.setEnabled(isEnabled);
			}
		});
		
		Label lhsLabel = new Label(container, SWT.NULL);
		lhsLabel.setText("Left hand side of Assertion: ");
		
		lhsText = new Text(container, SWT.BORDER | SWT.SINGLE);
		lhsText.setLayoutData(gd);
		lhsText.setEnabled(isEnabled);
		lhsText.setText("");
		lhsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e ) {
				fieldsModified();
			}
		});
		
		Label useMethodLabel = new Label(container, SWT.NULL);
		useMethodLabel.setText("Use method call as LHS: ");
		
		check2 = new Button(container, SWT.CHECK);
		check2.setEnabled(isEnabled);
		check2.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				
			}
			public void widgetSelected(SelectionEvent e) {
				if (check2.getSelection()) { // check box is ticked
					lhsText.setEnabled(false);
					lhsText.setText(methodCall);
				} else {
					lhsText.setEnabled(true);
					lhsText.setText("");
				}
				fieldsModified();
			}
		});
		
		Label compLabel = new Label(container, SWT.NULL);
		compLabel.setText("Comparison operator: ");
		
		operatorBox = new Combo(container, SWT.DROP_DOWN);
		operatorBox.setLayoutData(gd);
		operatorBox.setEnabled(isEnabled);		
				
		if (retType.equals("int") || retType.equals("double") || retType.equals("float")) {
			// assign numeric comparison operators to the combo box
			operatorBox.setItems(new String[] { "==", "!=", ">", ">=", "<", "<=" });
		} else {
			// assign object comparison operators to the combo box
			operatorBox.setItems(new String [] { "==" });
			
			// it would make sense to assign different comparison operators if the return type
			// was some complex object. This could possibly be done by iterating through the
			// return type and figuring out what function calls return simple values (ie. methods
			// with a return signature of int, bool, what have you) and populating the operations
			// list with those calls.  this is obviously work for the future.
		}
		
		operatorBox.select(0); // have the first item selected
		
		operatorBox.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent me) {
				fieldsModified();
			}
		});
		
		/*
		compText = new Text(container, SWT.BORDER | SWT.SINGLE);
		compText.setLayoutData(gd);
		compText.setEnabled(isEnabled);
		compText.setText("");
		compText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e ) {
				fieldsModified();
			}
		});
		*/
		Label rhsLabel = new Label(container, SWT.NULL);
		rhsLabel.setText("Right hand side of Assertion: ");
		
		rhsText = new Text(container, SWT.BORDER | SWT.SINGLE);
		rhsText.setLayoutData(gd);
		rhsText.setEnabled(isEnabled);
		rhsText.setText("");
		rhsText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e ) {
				fieldsModified();
			}
		});
		
		Label lb1 = new Label(container, SWT.NULL);
		lb1.setText("Assertion return value: ");
		
		Label lb2 = new Label(container, SWT.NULL); // fills empty space	
		
		Button trueButton = new Button(container, SWT.RADIO);
		Button falseButton = new Button(container, SWT.RADIO);
		
		trueButton.setText("True");
		trueButton.setEnabled(true);
		falseButton.setText("False");
		falseButton.setEnabled(true);		
		trueButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {				
			}
			public void widgetSelected(SelectionEvent e) {				
				expectedResult = true;
				fieldsModified();
			}
		});
		falseButton.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {				
			}
			public void widgetSelected(SelectionEvent e) {
				expectedResult = false;
				fieldsModified();
			}
		});
			
		Label lb = new Label(container, SWT.NULL); 
		lb.setText("Assertion Preview: ");
		
		previewText = new Text(container, SWT.BORDER | SWT.SINGLE);
		previewText.setLayoutData(gd);
		previewText.setEditable(false);
		previewText.setEnabled(isEnabled);
		previewText.setText("");
		
		setPageComplete(true);
		setControl(container);
	}
	
	protected String getLHS() {
		return lhsText.getText();
	}
	
	protected String getComparison() {
		return operatorBox.getText();
	}
	
	protected String getRHS() {
		return rhsText.getText();
	}
	
	protected boolean getExpectedResult() {
		return expectedResult;
	}
	
	protected boolean getAssertStatus() {
		return checkBox.getSelection();
	}
	
	/**
	 * Called to update the assertion preview whenever a change to the fields on this
	 * form is detected.
	 *
	 */
	private void fieldsModified() {				
		//System.out.println("fieldsModified triggered");
		//boolean x = trueButton.getSelection();
		boolean x = expectedResult;
		LogAssertion logAssert = new LogAssertion(lhsText.getText(), operatorBox.getText(), rhsText.getText(), x);
		//System.out.println(logAssert.to_String());
		if (isEnabled) {
			previewText.setText(logAssert.to_String());
		} else {
			previewText.setText("");
		}
	}
}
